from contextlib import contextmanager
from enum import IntEnum
import os 

class CERTIFIER_OPT(IntEnum):
    CERTIFIER_OPT_CFG_FILENAME         = 1
    CERTIFIER_OPT_AUTH_TYPE            = 2
    CERTIFIER_OPT_CERTIFIER_URL        = 3
    CERTIFIER_OPT_HTTP_TIMEOUT         = 4
    CERTIFIER_OPT_HTTP_CONNECT_TIMEOUT = 5
    CERTIFIER_OPT_INPUT_P12_PATH       = 6
    CERTIFIER_OPT_INPUT_P12_PASSWORD   = 7
    CERTIFIER_OPT_OUTPUT_P12_PATH      = 8
    CERTIFIER_OPT_OUTPUT_P12_PASSWORD  = 9
    CERTIFIER_OPT_CA_INFO              = 10
    CERTIFIER_OPT_CA_PATH              = 11
    CERTIFIER_OPT_CRT                  = 12
    CERTIFIER_OPT_PROFILE_NAME         = 13
    CERTIFIER_OPT_OPTIONS      = 14
    CERTIFIER_OPT_ECC_CURVE_ID = 15
    CERTIFIER_OPT_SYSTEM_ID    = 16
    CERTIFIER_OPT_FABRIC_ID    = 17
    CERTIFIER_OPT_PRODUCT_ID   = 18
    # 19 is unused
    CERTIFIER_OPT_LOG_FILENAME = 20
    CERTIFIER_OPT_LOG_LEVEL    = 21
    # 22 is unused
    CERTIFIER_OPT_AUTH_TOKEN  = 23
    CERTIFIER_OPT_OUTPUT_NODE = 24
    CERTIFIER_OPT_TARGET_NODE = 25
    CERTIFIER_OPT_ACTION      = 26
    CERTIFIER_OPT_INPUT_NODE  = 27
    CERTIFIER_OPT_NODE_ID     = 28
    CERTIFIER_OPT_AUTH_TAG_1  = 29
    CERTIFIER_OPT_DNS_SAN   = 30
    CERTIFIER_OPT_IP_SAN    = 31
    CERTIFIER_OPT_EMAIL_SAN = 32
    # 33 - 36 are unused
    CERTIFIER_OPT_LOG_MAX_SIZE = 37
    # 38,39 are unused
    # 40 - 43 are unused
    CERTIFIER_OPT_TRACKING_ID = 44
    CERTIFIER_OPT_SOURCE        = 45
    CERTIFIER_OPT_CN_PREFIX     = 46
    CERTIFIER_OPT_VALIDITY_DAYS = 47
    CERTIFIER_OPT_EXT_KEY_USAGE = 48
    CERTIFIER_OPT_DOMAIN        = 49
    CERTIFIER__OPT_LOG_FUNCTION = 50
    CERTIFIER_OPT_CERT_MIN_TIME_LEFT_S = 51
    CERTIFIER_OPT_BOOL_FIRST = 52
    CERTIFIER_OPT_DEBUG_HTTP = CERTIFIER_OPT_BOOL_FIRST
    CERTIFIER_OPT_TRACE_HTTP = 53  
    CERTIFIER_OPT_FORCE_REGISTRATION = 54
    CERTIFIER_OPT_MEASURE_PERFORMANCE = 55
    CERTIFIER_OPT_CERTIFICATE_LITE = 56
    CERTIFIER_OPT_MAC_ADDRESS = 57
    CERTIFIER_OPT_SIMULATION_CERT_EXP_DATE_BEFORE = 58
    CERTIFIER_OPT_SIMULATION_CERT_EXP_DATE_AFTER = 59
    CERTIFIER_OPT_AUTORENEW_INTERVAL = 60
    CERTIFIER_OPT_AUTORENEW_CERTS_PATH_LIST = 61
    CERTIFIER_OPT_USE_SCOPES = 62
    CERTIFIER_OPT_CERT_X509_OUT = 63
    CERTIFIER_OPT_MTLS_P12_PATH = 64
    CERTIFIER_OPT_MTLS_P12_PASSWORD = 65

class CERTIFIER_OPT_OPTION(IntEnum):
    CERTIFIER_OPTION_DEBUG_HTTP          = 1
    CERTIFIER_OPTION_TRACE_HTTP          = 2
    CERTIFIER_OPTION_FORCE_REGISTRATION  = 4
    CERTIFIER_OPTION_MEASURE_PERFORMANCE = 8
    CERTIFIER_OPTION_CERTIFICATE_LITE    = 16
    CERTIFIER_OPTION_USE_SCOPES          = 32
    # 64, 128, 256, 512, 1024 are unused

# certifier.py
VERY_SMALL_STRING_SIZE = 32

ALLOWABLE_CHARACTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnpqrstuvwxyz0123456879"

CERTIFIER_ERR_INIT_CERTIFIER = 1000
# CERTIFIER_ERR_INIT_MINER =     2000
CERTIFIER_ERR_INIT_SECURITY = 3000
CERTIFIER_ERR_INIT_CAMERA = 4000
CERTIFIER_ERR_INIT_MEMORY = 4500

CERTIFIER_ERR_DESTROY_CERTIFIER = 5000
# CERTIFIER_ERR_DESTROY_MINER     6000
CERTIFIER_ERR_DESTROY_SECURITY = 7000
CERTIFIER_ERR_DESTROY_CAMERA = 7500
CERTIFIER_ERR_DESTROY_LOG = 7800
CERTIFIER_ERR_DESTROY_PROPERTY = 7900

CERTIFIER_ERR_REGISTER_SECURITY_1 = 9000
CERTIFIER_ERR_REGISTER_SECURITY_5 = 9001
CERTIFIER_ERR_REGISTER_SECURITY_6 = 9002
CERTIFIER_ERR_REGISTER_SECURITY_7 = 9003
CERTIFIER_ERR_REGISTER_DELETE_PKCS12_1 = 9004
CERTIFIER_ERR_REGISTER_RENAME_PKCS12_1 = 9005
CERTIFIER_ERR_REGISTER_DELETE_PKCS12_2 = 9006
CERTIFIER_ERR_REGISTER_RENAME_PKCS12_2 = 9007
CERTIFIER_ERR_REGISTER_CERT_RENEWAL = 10000
CERTIFIER_ERR_REGISTER_SETUP = 11000
CERTIFIER_ERR_REGISTER_CERTIFIER_1 = 12000
CERTIFIER_ERR_REGISTER_MINER_1 = 13000
CERTIFIER_ERR_REGISTER_CRT_1 = 14000
CERTIFIER_ERR_REGISTER_CRT_2 = 15000
CERTIFIER_ERR_REGISTER_UNKNOWN = 16000

CERTIFIER_ERR_PROPERTY_SET = 27000
CERTIFIER_ERR_PROPERTY_SET_MEMORY = 27900

CERTIFIER_ERR_CREATE_NODE_ADDRESS_1 = 100100
CERTIFIER_ERR_CREATE_NODE_ADDRESS_2 = 100200

CERTIFIER_ERR_CREATE_CRT_1 = 100300
CERTIFIER_ERR_CREATE_CRT_2 = 100301
CERTIFIER_ERR_CREATE_CRT_3 = 100302
CERTIFIER_ERR_CREATE_CRT_4 = 100303
CERTIFIER_ERR_CREATE_CRT_5 = 100304
CERTIFIER_ERR_CREATE_CRT_6 = 100305

CERTIFIER_ERR_CREATE_X509_CERT_1 = 100400
CERTIFIER_ERR_CREATE_X509_CERT_2 = 100401
CERTIFIER_ERR_CREATE_X509_CERT_3 = 100402
CERTIFIER_ERR_CREATE_X509_CERT_4 = 100500
CERTIFIER_ERR_CREATE_X509_CERT_5 = 100600
CERTIFIER_ERR_CREATE_X509_CERT_6 = 100680
CERTIFIER_ERR_CREATE_X509_CERT_7 = 100690

CERTIFIER_ERR_EMPTY_OR_INVALID_PARAM_1 = 100800
CERTIFIER_ERR_EMPTY_OR_INVALID_PARAM_2 = 100801
CERTIFIER_ERR_EMPTY_OR_INVALID_PARAM_3 = 100802
CERTIFIER_ERR_GEN_1 = 100803
CERTIFIER_ERR_EMPTY_OR_INVALID_PARAM_5 = 100804

CERTIFIER_ERR_REVOKE_CERT_STATUS_1 = 110000

CERTIFIER_ERR_GET_CERT_STATUS_1 = (1 << 15)

CERTIFIER_ERR_RENEW_CERT_1 = 140000

CERTIFIER_ERR_PRINT_CERT_1 = 150000
CERTIFIER_ERR_PRINT_CERT_2 = 160000
CERTIFIER_ERR_PRINT_CERT_3 = 170000
CERTIFIER_ERR_PRINT_CERT_4 = 180000
CERTIFIER_ERR_PRINT_CERT_5 = 190000

CERTIFIER_ERR_INTERNAL = 1
CERTIFIER_ERR_INVALID_ARGUMENT = 2

CERTIFIER_ERR_GENERATE_CRT_NONCE = 1

CERTIFIER_ERR_SETUP_ECKEY_FAILURE = 1

# NOTUSED - will repurpose
CERTIFIER_ERR_SETUP_ECKEY_PUBLIC_WRITE_DER_FAILURE_1 = 2

CERTIFIER_ERR_SETUP_INTERNAL_NODE_ADDRESS_2 = 3
CERTIFIER_ERR_SETUP_EMPTY_FILENAME = 4
CERTIFIER_ERR_SETUP_EMPTY_PASSWORD = 5
CERTIFIER_ERR_SETUP_EMPTY_ECC_CURVE = 6

CERTIFIER_ERR_REGISTRATION_STATUS_X509_NONEXISTENT = (1 << 0)
CERTIFIER_ERR_REGISTRATION_STATUS_P12_NONEXISTENT = (1 << 1)
CERTIFIER_ERR_REGISTRATION_STATUS_CERTIFIER_ID_NONEXISTENT = (1 << 2)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_TIME_CHECK_1 = (1 << 3)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_EXPIRED_1 = (1 << 4)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_EXPIRED_2 = (1 << 5)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_ABOUT_TO_EXPIRE = (1 << 6)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_TIME_VALID = (1 << 7)

CERTIFIER_ERR_GET_CERT_STATUS_UNKNOWN = (1 << 9)
CERTIFIER_ERR_GET_CERT_STATUS_REVOKED = (1 << 10)
CERTIFIER_ERR_GET_CERT_STATUS_GOOD = (1 << 11)

PEM_BEGIN_CERTIFICATE = "-----BEGIN CERTIFICATE-----"
PEM_BEGIN_CERTIFICATE_LENGTH = 27

PEM_END_CERTIFICATE = "-----END CERTIFICATE-----"
PEM_END_CERTIFICATE_LENGTH = 25
# END certifier.py

# certifier_internal.h

XTRA_SMALL_STRING_SIZE = 8
VERY_SMALL_STRING_SIZE = 32
SMALL_STRING_SIZE = 64
MEDIUM_STRING_SIZE = 256
LARGE_STRING_SIZE = 1024
VERY_LARGE_STRING_SIZE = 2048
MAX_STACK_SIZE = 10000

# end certifier_internal.h

# property.py

# from property_internal.h
CERTIFIER_ERR_PROPERTY_SET_1 = 1
CERTIFIER_ERR_PROPERTY_SET_2 = 2
CERTIFIER_ERR_PROPERTY_SET_3 = 3
CERTIFIER_ERR_PROPERTY_SET_4 = 4
CERTIFIER_ERR_PROPERTY_SET_5 = 5
CERTIFIER_ERR_PROPERTY_SET_6 = 6
CERTIFIER_ERR_PROPERTY_SET_7 = 7
CERTIFIER_ERR_PROPERTY_SET_8 = 8
CERTIFIER_ERR_PROPERTY_SET_9 = 9
CERTIFIER_ERR_PROPERTY_SET_10 = 10
CERTIFIER_ERR_PROPERTY_SET_11 = 11

CERTIFIER_ERR_PROPERTY_GET_1 = "CERTIFIER_ERR_PROPERTY_GET_1"
CERTIFIER_ERR_PROPERTY_GET_2 = "CERTIFIER_ERR_PROPERTY_GET_2"
# END property_internal.h

DEFAULT_LOG_LEVEL = 4
DEFAULT_LOG_MAX_SIZE = 5000000
DEFAULT_HTTP_TIMEOUT = 15
DEFAULT_HTTP_CONNECT_TIMEOUT = 15
DEFAULT_ECC_CURVE_ID = "prime256v1"
DEFAULT_OUTPUT_P12_PATH = "output.p12"
DEFAULT_AUTH_TYPE = "X509"
DEFAULT_CA_PATH = "/etc/ssl/certs"
DEFAULT_USER_CA_PATH = "/usr/local/etc/certfier"
DEFAULT_GLOBAL_CA_PATH = "/etc/certifier"
DEFAULT_CURDIR_CA_PATH = "."
DEFAULT_CERTIFER_URL = "https://certifier.xpki.io/v1/certifier"
DEFAULT_PROFILE_NAME = "XFN_Matter_OP_Class_3_ICA"
DEFAULT_CERT_MIN_TIME_LEFT_S = 90 * 24 * 60 * 60
DEFAULT_OPT_SOURCE = "unset-libcertifier-c-native"
DEFAULT_PRODUCT_ID = "1101"
DEFAULT_AUTORENEW_INTERVAL = 86400
DEFAULT_AUTORENEW_CERTS_PATH = "~/.libcertifier"
# END property.py

# xpki_client.py
SYSTEM_ID_SIZE = 40
CERTIFIER_STATIC_URL =  "https://cert-static.xpki.io/v1/certifier"
DEFAULT_CERTIFER_URL = "https://certifier.xpki.io/v1/certifier"
# END xpki_client.py

# security.h
VALIDATE_CERT = 0x1
VALIDATE_SIGNATURE = 0x2

CERTIFIER_ERR_SECURITY_ENCODE_1 = 1
CERTIFIER_ERR_SECURITY_ENCODE_2 = 2
CERTIFIER_ERR_SECURITY_ENCODE_3 = 3
CERTIFIER_ERR_SECURITY_ENCODE_4 = 5

CERTIFIER_SHA1_DIGEST_LENGTH = 20
# END security.h

def base64_encode_len(len: int):
    return (((len + 2) / 3) * 4) + 1

@contextmanager
def change_dir(destination):
    initial_dir = os.getcwd()

    try:
        os.chdir(destination)
        yield
    finally:
        os.chdir(initial_dir)


'''
revised error code mapping series below
'''

certifier_error = 9000
property_error = 8000

get_cert_error = 1000
get_crt_token_error = 2000
get_cert_status_error = 3000
get_cert_validity_error = 4000
renew_cert_error = 5000
print_cert_error = 6000
revoke_error = 7000